@@ -42,6 +42,30 @@ namespace Test {
         }
         return true;
     }
+
+    inline bool compareXml(const QString &refFile, const QByteArray &output, const QByteArray &ref)
+    {
+        if (output != ref) {
+            QFile failFile(refFile + QLatin1String(".fail"));
+            failFile.open(QFile::WriteOnly);
+            failFile.write(output);
+            failFile.close();
+
+            QProcess proc;
+            proc.setProcessChannelMode(QProcess::ForwardedChannels);
+            proc.start(QStringLiteral("diff"), {QStringLiteral("-u"), refFile, failFile.fileName()});
+            if (!proc.waitForFinished()) {
+                // e.g. Windows CI not having diff
+                qDebug() << "Actual:";
+                qDebug().noquote() << output;
+                qDebug() << "Expected:";
+                qDebug().noquote() << ref;
+            }
+            return false;
+        }
+        return true;
+    }
+
 }
 
 #endif
