@@ -5,6 +5,8 @@
 */
 
 #include "journey.h"
+
+#include "identifier_p.h"
 #include "journeyutil_p.h"
 #include "json_p.h"
 #include "datatypes_p.h"
@@ -17,6 +19,8 @@
 #include "rentalvehicleutil_p.h"
 #include "stopover.h"
 
+#include <KLocalizedString>
+
 #include <QDebug>
 #include <QVariant>
 
@@ -28,6 +32,8 @@ namespace KPublicTransport {
 class JourneySectionPrivate : public QSharedData
 {
 public:
+    [[nodiscard]] bool isValidIndex(qsizetype idx) const;
+
     JourneySection::Mode mode = JourneySection::Invalid;
     QDateTime scheduledDepartureTime;
     QDateTime expectedDepartureTime;
@@ -53,6 +59,7 @@ public:
     Vehicle arrivalVehicleLayout;
     Platform arrivalPlatformLayout;
     IndividualTransport individualTransport;
+    IdentifierSet ids;
 };
 
 class JourneyPrivate : public QSharedData
@@ -82,6 +89,11 @@ KPUBLICTRANSPORT_MAKE_PROPERTY(JourneySection, Vehicle, arrivalVehicleLayout, se
 KPUBLICTRANSPORT_MAKE_PROPERTY(JourneySection, Platform, arrivalPlatformLayout, setArrivalPlatformLayout)
 KPUBLICTRANSPORT_MAKE_PROPERTY(JourneySection, KPublicTransport::IndividualTransport, individualTransport, setIndividualTransport)
 
+bool JourneySectionPrivate::isValidIndex(qsizetype idx) const
+{
+    return idx >= 0 && idx <= (qsizetype)(intermediateStops.size()) + 1;
+}
+
 bool JourneySection::hasExpectedDepartureTime() const
 {
     return d->expectedDepartureTime.isValid();
@@ -264,6 +276,7 @@ Stopover JourneySection::departure() const
     dep.setDisruptionEffect(disruptionEffect());
     dep.setVehicleLayout(departureVehicleLayout());
     dep.setPlatformLayout(departurePlatformLayout());
+    dep.setLoadInformation(std::vector<LoadInfo>(loadInformation()));
     return dep;
 }
 
@@ -276,6 +289,9 @@ void JourneySection::setDeparture(const Stopover &departure)
     setExpectedDeparturePlatform(departure.expectedPlatform());
     setDeparturePlatformLayout(departure.platformLayout());
     setDepartureVehicleLayout(departure.vehicleLayout());
+    if (!departure.loadInformation().empty()) {
+        setLoadInformation(std::vector<LoadInfo>(departure.loadInformation()));
+    }
     if (departure.disruptionEffect() == Disruption::NoService) {
         setDisruptionEffect(departure.disruptionEffect());
     }
@@ -427,14 +443,21 @@ void JourneySection::setLoadInformation(std::vector<LoadInfo> &&loadInfo)
     d->loadInformation = std::move(loadInfo);
 }
 
-QVariantList JourneySection::loadInformationVariant() const
+QList<LoadInfo> JourneySection::loadInformationList() const
 {
-    QVariantList l;
-    l.reserve(d->loadInformation.size());
-    std::transform(d->loadInformation.begin(), d->loadInformation.end(), std::back_inserter(l), [](const auto &load) { return QVariant::fromValue(load); });
+    QList<LoadInfo> l;
+    l.reserve((qsizetype)d->loadInformation.size());
+    std::copy(d->loadInformation.begin(), d->loadInformation.end(), std::back_inserter(l));
     return l;
 }
 
+void JourneySection::setLoadInformationList(const QList<LoadInfo> &loadInfo)
+{
+    d->loadInformation.clear();
+    d->loadInformation.reserve(loadInfo.size());
+    std::copy(loadInfo.begin(), loadInfo.end(), std::back_inserter(d->loadInformation));
+}
+
 const std::vector<KPublicTransport::Feature>& JourneySection::features() const
 {
     return d->departureVehicleLayout.features();
@@ -493,6 +516,164 @@ QString JourneySection::modeIconName(JourneySection::Mode mode)
     return u"question"_s;
 }
 
+QString JourneySection::label() const
+{
+    switch (mode()) {
+    case JourneySection::Invalid:
+        break;
+    case JourneySection::PublicTransport:
+        return route().line().name();
+    case JourneySection::Walking:
+        return i18nc("mode of individual transport", "Walk");
+    case JourneySection::Waiting:
+        return i18n("Wait");
+    case JourneySection::Transfer:
+        break; // ?
+    case JourneySection::RentedVehicle:
+        return rentalVehicle().label();
+    case JourneySection::IndividualTransport:
+        return individualTransport().label();
+    }
+
+    return {};
+}
+
+Load::Category JourneySection::maximumOccupancy() const
+{
+    return std::accumulate(d->loadInformation.begin(), d->loadInformation.end(), Load::Unknown, [](auto l, const auto &info) {
+        return std::max(l, info.load());
+    });
+}
+
+QString JourneySection::identifier(QAnyStringView identifierType) const
+{
+    return d->ids.identifier(identifierType);
+}
+
+bool JourneySection::hasIdentifier(QAnyStringView identifierType) const
+{
+    return d->ids.hasIdentifier(identifierType);
+}
+
+void JourneySection::setIdentifier(const QString &identifierType, const QString &id)
+{
+    d.detach();
+    d->ids.setIdentifier(identifierType, id);
+}
+
+bool JourneySection::hasIdentifiers() const
+{
+    return !d->ids.isEmpty();
+}
+
+Stopover JourneySection::stopover(qsizetype idx) const
+{
+    if (!d->isValidIndex(idx)) {
+        return {};
+    }
+
+    if (idx == 0) {
+        return departure();
+    }
+    if (idx <= (qsizetype)d->intermediateStops.size()) {
+        return d->intermediateStops[idx - 1];
+    }
+    return arrival();
+}
+
+void JourneySection::setStopovver(qsizetype idx, const Stopover &stop)
+{
+    if (!d->isValidIndex(idx)) {
+        return;
+    }
+
+    if (idx == 0) {
+        setDeparture(stop);
+    } else if (idx <= (qsizetype)d->intermediateStops.size()) {
+        d.detach();
+        d->intermediateStops[idx - 1] = stop;
+    } else {
+        setArrival(stop);
+    }
+}
+
+qsizetype JourneySection::indexOfStopover(const Stopover &stop) const
+{
+    if (Stopover::isSame(departure(), stop)) {
+        return 0;
+    }
+    if (const auto it = std::ranges::find_if(d->intermediateStops, [&stop](const auto &s) { return Stopover::isSame(s, stop); }); it != d->intermediateStops.end()) {
+        return std::distance(d->intermediateStops.begin(), it) + 1;
+    }
+    if (Stopover::isSame(arrival(), stop)) {
+        return (qsizetype)d->intermediateStops.size() + 1;
+    }
+    return -1;
+}
+
+JourneySection JourneySection::subsection(qsizetype begin, qsizetype end) const
+{
+    if (!d->isValidIndex(begin) || !d->isValidIndex(end) || end <= begin) {
+        return {};
+    }
+
+    auto partialTrip = *this;
+    if (begin > 0) {
+        partialTrip.setDeparture(stopover(begin));
+    }
+    if (end <= (qsizetype)d->intermediateStops.size()) {
+        partialTrip.setArrival(stopover(end));
+    }
+
+    const auto beginIt = d->intermediateStops.begin() + std::max<qsizetype>(0, begin);
+    const auto endIt = beginIt + std::max<qsizetype>(0, end - begin - 1);
+    partialTrip.setIntermediateStops({beginIt, endIt});
+
+    auto path = d->path;
+    auto pathSections = path.takeSections();
+
+    // find the closest points on the path to the departure/arrival locations
+    // we have to expect aribitrarly broken/imprecise data here...
+    qsizetype beginSection = -1, beginPoint = -1, endSection = -1, endPoint = -1;
+    double beginMinDist = std::numeric_limits<double>::max(), endMinDist = std::numeric_limits<double>::max();
+    for (std::size_t i = 0; i < pathSections.size(); ++i) {
+        const auto poly = pathSections[i].path();
+        for (qsizetype j = 0; j < poly.size(); ++j) {
+            const auto p = poly[j];
+            if (const auto d = Location::distance(p.y(), p.x(), partialTrip.from().latitude(), partialTrip.from().longitude()); d <beginMinDist) {
+                beginSection = (qsizetype)i;
+                beginPoint = j;
+                beginMinDist = d;
+            }
+            if (const auto d = Location::distance(p.y(), p.x(), partialTrip.to().latitude(), partialTrip.to().longitude()); d <endMinDist) {
+                endSection = (qsizetype)i;
+                endPoint = j;
+                endMinDist = d;
+            }
+        }
+    }
+
+    // if we found something, truncate path accordingly
+    if (std::tie(beginSection, beginPoint) < std::tie(endSection, endPoint)) {
+        // start cutting from the end, as otherwise the end indices become invalid!
+        pathSections.erase(pathSections.begin() + endSection + 1, pathSections.end());
+        pathSections.erase(pathSections.begin(), pathSections.begin() + beginSection);
+
+        auto poly = pathSections.back().path();
+        poly.erase(poly.begin() + endPoint + 1, poly.end());
+        pathSections.back().setPath(poly);
+
+        poly = pathSections.front().path();
+        poly.erase(poly.begin(), poly.begin() + beginPoint);
+        pathSections.front().setPath(poly);
+
+        path.setSections(std::move(pathSections));
+        partialTrip.setPath(path);
+    }
+
+    return partialTrip;
+}
+
 void JourneySection::applyMetaData(bool download)
 {
     if (!from().hasCoordinate() || mode() != JourneySection::PublicTransport) {
@@ -518,6 +699,12 @@ bool JourneySection::isSame(const JourneySection &lhs, const JourneySection &rhs
         return false;
     }
 
+    switch (lhs.d->ids.compare(rhs.d->ids)) {
+        case IdentifierSet::NotEqual: return false;
+        case IdentifierSet::Equal: // same trip id can still mean diffierent departure/arrival stops
+        case IdentifierSet::NoIntersection: break;
+    }
+
     // we have N criteria to compare here, with 3 possible results:
     // - equal
     // - similar-ish, unknwon, or at least not conflicting
@@ -556,6 +743,7 @@ JourneySection JourneySection::merge(const JourneySection &lhs, const JourneySec
 {
     using namespace MergeUtil;
     auto res = lhs;
+    res.d->ids.merge(rhs.d->ids);
     res.setScheduledDepartureTime(mergeDateTimeEqual(lhs.scheduledDepartureTime(), rhs.scheduledDepartureTime()));
     res.setExpectedDepartureTime(mergeDateTimeMax(lhs.expectedDepartureTime(), rhs.expectedDepartureTime()));
     res.setScheduledArrivalTime(mergeDateTimeMax(lhs.scheduledArrivalTime(), rhs.scheduledArrivalTime()));
@@ -576,7 +764,7 @@ JourneySection JourneySection::merge(const JourneySection &lhs, const JourneySec
 
     res.setDisruptionEffect(std::max(lhs.disruptionEffect(), rhs.disruptionEffect()));
     res.setNotes(NotesUtil::mergeNotes(lhs.notes(), rhs.notes()));
-    res.setDistance(std::max(lhs.distance(), rhs.distance()));
+    res.setDistance(std::max(lhs.d->distance, rhs.d->distance));
 
     if (lhs.intermediateStops().size() == rhs.intermediateStops().size()) {
         auto stops = res.takeIntermediateStops();
@@ -585,6 +773,10 @@ JourneySection JourneySection::merge(const JourneySection &lhs, const JourneySec
             stops[i].setRoute(res.route());
         }
         res.setIntermediateStops(std::move(stops));
+    } else if (lhs.intermediateStops().empty() && !rhs.intermediateStops().empty()) {
+        res.setIntermediateStops(std::vector<Stopover>(rhs.intermediateStops()));
+    } else if (!lhs.intermediateStops().empty() && rhs.intermediateStops().empty()) {
+        res.setIntermediateStops(std::vector<Stopover>(lhs.intermediateStops()));
     }
 
     res.d->co2Emission = std::max(lhs.d->co2Emission, rhs.d->co2Emission);
@@ -604,6 +796,9 @@ JourneySection JourneySection::merge(const JourneySection &lhs, const JourneySec
 QJsonObject JourneySection::toJson(const JourneySection &section)
 {
     auto obj = Json::toJson(section);
+    if (!section.d->ids.isEmpty()) {
+        obj.insert("identifiers"_L1, section.d->ids.toJson());
+    }
     if (section.mode() != Waiting) {
         const auto fromObj = Location::toJson(section.from());
         if (!fromObj.empty()) {
@@ -626,6 +821,9 @@ QJsonObject JourneySection::toJson(const JourneySection &section)
             obj.insert(QLatin1String("load"), LoadInfo::toJson(section.loadInformation()));
         }
     }
+    if (section.d->distance <= 0) {
+        obj.remove("distance"_L1);
+    }
     if (section.d->co2Emission < 0) {
         obj.remove(QLatin1String("co2Emission"));
     }
@@ -654,7 +852,7 @@ QJsonObject JourneySection::toJson(const JourneySection &section)
         obj.insert(QLatin1String("individualTransport"), IndividualTransport::toJson(section.individualTransport()));
     }
 
-    if (obj.size() <= 3) { // only the disruption and mode enums and distance, ie. this is an empty object
+    if (obj.size() <= 2) { // only the disruption and mode enums, ie. this is an empty object
         return {};
     }
     return obj;
@@ -668,6 +866,7 @@ QJsonArray JourneySection::toJson(const std::vector<JourneySection> &sections)
 JourneySection JourneySection::fromJson(const QJsonObject &obj)
 {
     auto section = Json::fromJson<JourneySection>(obj);
+    section.d->ids.fromJson(obj.value("identifiers"_L1).toObject());
     section.setFrom(Location::fromJson(obj.value(QLatin1String("from")).toObject()));
     section.setTo(Location::fromJson(obj.value(QLatin1String("to")).toObject()));
     section.setRoute(Route::fromJson(obj.value(QLatin1String("route")).toObject()));
@@ -709,14 +908,21 @@ void Journey::setSections(std::vector<JourneySection> &&sections)
     d->sections = std::move(sections);
 }
 
-QVariantList Journey::sectionsVariant() const
+QList<JourneySection> Journey::sectionsList() const
 {
-    QVariantList l;
-    l.reserve(d->sections.size());
-    std::transform(d->sections.begin(), d->sections.end(), std::back_inserter(l), [](const auto &sec) { return QVariant::fromValue(sec); });
+    QList<JourneySection> l;
+    l.reserve((qsizetype)d->sections.size());
+    std::copy(d->sections.begin(), d->sections.end(), std::back_inserter(l));
     return l;
 }
 
+void Journey::setSectionsList(const QList<JourneySection> &sections)
+{
+    d->sections.clear();
+    d->sections.reserve(sections.size());
+    std::copy(sections.begin(), sections.end(), std::back_inserter(d->sections));
+}
+
 QDateTime Journey::scheduledDepartureTime() const
 {
     if (!d->sections.empty()) {
@@ -792,6 +998,13 @@ int Journey::co2Emission() const
     return std::accumulate(d->sections.begin(), d->sections.end(), 0, [](auto co2, const auto &jny) { return co2 + std::max(0, jny.co2Emission()); });
 }
 
+Load::Category Journey::maximumOccupancy() const
+{
+    return std::accumulate(d->sections.begin(), d->sections.end(), Load::Unknown, [](auto l, const auto &jny) {
+        return std::max(l, jny.maximumOccupancy());
+    });
+}
+
 void Journey::applyMetaData(bool download)
 {
     for (auto &sec : d->sections) {
