@@ -102,42 +102,57 @@ QString Line::modeIconName(Line::Mode mode)
         case Line::Unknown:
             break;
         case Line::Air:
-            return u"qrc:///org.kde.kpublictransport/assets/images/flight.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-flight.svg"_s;
         case Line::Boat:
         case Line::Ferry:
-            return u"qrc:///org.kde.kpublictransport/assets/images/ferry.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-ferry.svg"_s;
         case Line::Bus:
-            return u"qrc:///org.kde.kpublictransport/assets/images/bus.svg"_s;
-        case Line::BusRapidTransit:
+        case Line::Shuttle:
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-bus.svg"_s;
         case Line::Coach:
-            return u"qrc:///org.kde.kpublictransport/assets/images/coach.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-bus-longdistance.svg"_s;
         case Line::Funicular:
-            return u"qrc:///org.kde.kpublictransport/assets/images/funicular.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-funicular.svg"_s;
         case Line::LocalTrain:
         case Line::Train:
-            return u"qrc:///org.kde.kpublictransport/assets/images/train.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-train.svg"_s;
         case Line::LongDistanceTrain:
-            return u"qrc:///org.kde.kpublictransport/assets/images/longdistancetrain.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-train-longdistance.svg"_s;
         case Line::Metro:
-            return u"qrc:///org.kde.kpublictransport/assets/images/subway.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-subway.svg"_s;
         case Line::RailShuttle:
         case Line::RapidTransit:
-            return u"qrc:///org.kde.kpublictransport/assets/images/rapidtransit.svg"_s;
-        case Line::Shuttle:
-            return u"qrc:///org.kde.kpublictransport/assets/images/shuttle.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-rapidtransit.svg"_s;
         case Line::Taxi:
-            return u"qrc:///org.kde.kpublictransport/assets/images/taxi.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-taxi.svg"_s;
         case Line::Tramway:
-            return u"qrc:///org.kde.kpublictransport/assets/images/tramway.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-tram.svg"_s;
         case Line::RideShare:
-            return u"qrc:///org.kde.kpublictransport/assets/images/car.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-car.svg"_s;
         case Line::AerialLift:
-            return u"qrc:///org.kde.kpublictransport/assets/images/aeriallift.svg"_s;
+            return u"qrc:///org.kde.kpublictransport/assets/images/transport-mode-aeriallift.svg"_s;
     }
 
     return u"question"_s;
 }
 
+bool Line::modeIsRailBound(Line::Mode mode)
+{
+    switch (mode) {
+        case Line::Train:
+        case Line::Funicular:
+        case Line::LocalTrain:
+        case Line::LongDistanceTrain:
+        case Line::Metro:
+        case Line::RailShuttle:
+        case Line::RapidTransit:
+        case Line::Tramway:
+            return true;
+        default:
+            return false;
+    }
+}
+
 QString Line::iconName() const
 {
     if (hasLogo()) {
