@@ -141,11 +141,11 @@ void Stopover::setLoadInformation(std::vector<LoadInfo> &&loadInfo)
     d->loadInformation = std::move(loadInfo);
 }
 
-QVariantList Stopover::loadInformationVariant() const
+QList<LoadInfo> Stopover::loadInformationList() const
 {
-    QVariantList l;
-    l.reserve(d->loadInformation.size());
-    std::transform(d->loadInformation.begin(), d->loadInformation.end(), std::back_inserter(l), [](const auto &load) { return QVariant::fromValue(load); });
+    QList<LoadInfo> l;
+    l.reserve((qsizetype)d->loadInformation.size());
+    std::copy(d->loadInformation.begin(), d->loadInformation.end(), std::back_inserter(l));
     return l;
 }
 
@@ -165,6 +165,13 @@ void Stopover::setFeatures(std::vector<KPublicTransport::Feature> &&features)
     d->vehicleLayout.setFeatures(std::move(features));
 }
 
+Load::Category Stopover::maximumOccupancy() const
+{
+    return std::accumulate(d->loadInformation.begin(), d->loadInformation.end(), Load::Unknown, [](auto l, const auto &info) {
+        return std::max(l, info.load());
+    });
+}
+
 void Stopover::applyMetaData(bool download)
 {
     auto line = d->route.line();
@@ -209,7 +216,7 @@ Stopover Stopover::merge(const Stopover &lhs, const Stopover &rhs)
     using namespace MergeUtil;
     stopover.setScheduledDepartureTime(mergeDateTimeEqual(lhs.scheduledDepartureTime(), rhs.scheduledDepartureTime()));
     stopover.setExpectedDepartureTime(mergeDateTimeMax(lhs.expectedDepartureTime(), rhs.expectedDepartureTime()));
-    stopover.setScheduledArrivalTime(mergeDateTimeEqual(lhs.scheduledArrivalTime(), rhs.scheduledArrivalTime()));
+    stopover.setScheduledArrivalTime(mergeDateTimeMax(lhs.scheduledArrivalTime(), rhs.scheduledArrivalTime()));
     stopover.setExpectedArrivalTime(mergeDateTimeMax(lhs.expectedArrivalTime(), rhs.expectedArrivalTime()));
 
     if (stopover.scheduledPlatform().isEmpty() && !rhs.scheduledPlatform().isEmpty()) {
