@@ -182,30 +182,7 @@ void JourneyReplyPrivate::postProcessJourneys(std::vector<Journey> &journeys)
         sections.erase(std::remove_if(sections.begin(), sections.end(), isPointlessSection), sections.end());
 
         for (auto &section : sections) {
-            if (!section.from().hasCoordinate() || !section.to().hasCoordinate() || section.path().isEmpty()) {
-                continue;
-            }
-
-            // remove implausible paths
-            const auto pointDist = Location::distance(section.from(), section.to());
-            const auto pathDist = section.path().distance();
-            if (pathDist > pointDist * 10) {
-                qCDebug(Log) << "Dropping implausibly long path:" << pointDist << pathDist;
-                section.setPath({});
-            }
-
-            // filter spikes found in rail paths in nearly all backends and GTFS shapes
-            if (section.mode() == JourneySection::PublicTransport && Line::modeIsRailBound(section.route().line().mode())) {
-                auto path = section.path();
-                auto pathSecs = path.takeSections();
-                for (auto &pathSec : pathSecs) {
-                    QPolygonF p = pathSec.path();
-                    PathFilter::removeSpikes(p, 55.0);
-                    pathSec.setPath(p);
-                }
-                path.setSections(std::move(pathSecs));
-                section.setPath(path);
-            }
+            JourneyUtil::postProcessPath(section);
         }
 
         journey.setSections(std::move(sections));
@@ -350,3 +327,5 @@ void JourneyReply::addError(const AbstractBackend *backend, Reply::Error error,
     }
     Reply::addError(error, errorMsg);
 }
+
+#include "moc_journeyreply.cpp"
