@@ -29,6 +29,8 @@ class LocationRequest;
 class ManagerPrivate;
 class StopoverReply;
 class StopoverRequest;
+class TripReply;
+class TripRequest;
 class VehicleLayoutReply;
 class VehicleLayoutRequest;
 
@@ -54,7 +56,7 @@ class KPUBLICTRANSPORT_EXPORT Manager : public QObject
     Q_PROPERTY(bool backendsEnabledByDefault READ backendsEnabledByDefault WRITE setBackendsEnabledByDefault NOTIFY configurationChanged)
 
     /** QML-compatible access to backends(). */
-    Q_PROPERTY(QVariantList backends READ backendsVariant CONSTANT)
+    Q_PROPERTY(QVariantList backends READ backendsVariant NOTIFY backendsChanged)
 
 public:
     explicit Manager(QObject *parent = nullptr);
@@ -67,37 +69,48 @@ public:
     void setNetworkAccessManager(QNetworkAccessManager *nam);
 
     /** Returns whether access to insecure backends is allowed. */
-    bool allowInsecureBackends() const;
+    [[nodiscard]] bool allowInsecureBackends() const;
     /** Allow usage of insecure backends, that is services not using
      *  transport encryption.
      */
     void setAllowInsecureBackends(bool insecure);
 
     /** Query a journey. */
-    JourneyReply* queryJourney(const JourneyRequest &req) const;
+    [[nodiscard]] JourneyReply* queryJourney(const JourneyRequest &req) const;
 
     /** Query arrivals or departures from a specific station. */
-    StopoverReply* queryStopover(const StopoverRequest &req) const;
+    [[nodiscard]] StopoverReply* queryStopover(const StopoverRequest &req) const;
 
     /** Query location information based on coordinates or (parts of) the name. */
-    LocationReply* queryLocation(const LocationRequest &req) const;
+    [[nodiscard]] LocationReply* queryLocation(const LocationRequest &req) const;
+
+    /** Query trip information.
+     *  That is a specific run of a vehicle on a route.
+     *  When there's a backend supporting trip queries this returns the full trip.
+     *  For backends not supporting this, this internally falls back to a journey
+     *  query and thus might return only a subset of the trip, based on the JourneySection
+     *  used in the request.
+     *
+     *  @since 25.04
+     */
+    Q_INVOKABLE [[nodiscard]] KPublicTransport::TripReply* queryTrip(const TripRequest &req) const;
 
     /** Query vehicle and platform layout information.
      *  This is only available for some trains and some operators, so be prepared
      *  for empty results.
      */
-    VehicleLayoutReply* queryVehicleLayout(const VehicleLayoutRequest &req) const;
+    [[nodiscard]] VehicleLayoutReply* queryVehicleLayout(const VehicleLayoutRequest &req) const;
 
     /** Returns all static attribution information, as well as all dynamic ones
      *  found in the cache or accumulated during the lifetime of this instance.
      */
-    const std::vector<Attribution>& attributions() const;
+    [[nodiscard]] const std::vector<Attribution>& attributions() const;
 
     /** Returns information about all available backends. */
-    const std::vector<Backend>& backends() const;
+    [[nodiscard]] const std::vector<Backend>& backends() const;
 
     /** Returns whether the use of the backend with a given identifier is enabled. */
-    Q_INVOKABLE bool isBackendEnabled(const QString &backendId) const;
+    Q_INVOKABLE [[nodiscard]] bool isBackendEnabled(const QString &backendId) const;
     /** Sets whether the backend with the given identifier should be used.
      *  @note If allowInsecureBackends() is @c false, this has precedence.
      */
@@ -106,7 +119,7 @@ public:
     /** Returns the identifiers of explicitly enabled backends.
      *  Use this for persisting the settings, not for checking for enabled backends.
      */
-    QStringList enabledBackends() const;
+    [[nodiscard]] QStringList enabledBackends() const;
     /** Sets the explicitly enabled backends.
      *  Use for restoring persisted settings.
      */
@@ -114,7 +127,7 @@ public:
     /** Returns the identifiers of explicitly disabled backends.
      *  Use this for persisting settings, not for checking for disabled backends.
      */
-    QStringList disabledBackends() const;
+    [[nodiscard]] QStringList disabledBackends() const;
     /** Sets the explicitly disabled backends.
      *  Use for restoring persisted settings.
      */
@@ -123,18 +136,26 @@ public:
      * Returns wheter backends are enabled by default.
      * Defaults to true.
      */
-    bool backendsEnabledByDefault() const;
+    [[nodiscard]] bool backendsEnabledByDefault() const;
     /**
      * Set wheter backends are enabled by default.
      */
     void setBackendsEnabledByDefault(bool byDefault);
 
+public Q_SLOTS:
+    /** Reload backend configuration.
+     *  Can be used when on-disk configuration has been changed.
+     *  Automatically called on language changes.
+     */
+    void reload();
 
 Q_SIGNALS:
     void attributionsChanged();
     void configurationChanged();
+    void backendsChanged();
 
 private:
+    bool eventFilter(QObject *object, QEvent *event) override;
     Q_DECL_HIDDEN QVariantList attributionsVariant() const;
     Q_DECL_HIDDEN QVariantList backendsVariant() const;
 
