@@ -18,7 +18,6 @@
 namespace KPublicTransport {
 
 class Stopover;
-class JourneySection;
 class VehicleLayoutRequestPrivate;
 
 /** Describes a query for vehicle layout information. */
@@ -28,8 +27,6 @@ class KPUBLICTRANSPORT_EXPORT VehicleLayoutRequest
 
     /** The stopover vehicle and platform layout information are requested for. */
     KPUBLICTRANSPORT_PROPERTY(KPublicTransport::Stopover, stopover, setStopover)
-    /** @deprecated use stopover .*/
-    Q_PROPERTY(KPublicTransport::Stopover departure READ departure WRITE setDeparture)
 
 public:
     /** Creates a vehicle layout request for the given stopover. */
@@ -51,9 +48,6 @@ public:
      */
     void setBackendIds(const QStringList &backendIds);
 
-    [[deprecated("use stopover()")]] Stopover departure() const;
-    [[deprecated("use setStopover()")]] void setDeparture(const Stopover &departure);
-
     ///@cond internal
     static QJsonObject toJson(const VehicleLayoutRequest &req);
     ///@endcond
